//$Id: HotelSearchingAction.java 8998 2008-09-16 03:08:11Z shane.bryzak@jboss.com $
package org.jboss.seam.example.booking;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.log.LogProvider;
import org.jboss.seam.log.Logging;

@Stateful
@Name("hotelSearch")
@Scope(ScopeType.SESSION)
@Restrict("#{identity.loggedIn}")
public class HotelSearchingAction implements HotelSearching
{
    @PersistenceContext
    private EntityManager em;
    
    private String searchString;
    private int pageSize = 10;
    private int page;
    private boolean nextPageAvailable;
   
    @DataModel
    private List<Hotel> hotels;

    private static LogProvider log = Logging.getLogProvider(HotelSearchingAction.class);
   
    public void find() 
    {
        log.trace("find");
        page = 0;
        queryHotels();
    }

    public void nextPage() 
    {
        log.trace("nextPage");
        page++;
        queryHotels();
    }
    
    private void queryHotels() {
        log.trace("queryHotels pre query");
        List<Hotel> results = em.createQuery("select h from Hotel h where lower(h.name) like #{pattern} or lower(h.city) like #{pattern} or lower(h.zip) like #{pattern} or lower(h.address) like #{pattern}")
                                .setMaxResults(pageSize+1)
                                .setFirstResult(page * pageSize)
                                .getResultList();

        // List<Hotel> results = new ArrayList<Hotel>();

        log.trace("queryHotels after query");

        nextPageAvailable = results.size() > pageSize;
        if (nextPageAvailable) 
        {
            hotels = new ArrayList<Hotel>(results.subList(0,pageSize));
        } else {
            hotels = results;
        }
    }

    public boolean isNextPageAvailable()
    {
        log.trace("isNextPageAvailable");
        return nextPageAvailable;
    }
   
   public int getPageSize() {
      log.trace("getPageSize");
      return pageSize;
   }
   
   public void setPageSize(int pageSize) {
      log.trace("setPageSize");
      this.pageSize = pageSize;
   }
   
   @Factory(value="pattern", scope=ScopeType.EVENT)
   public String getSearchPattern()
   {
      log.trace("getSearchPattern");
      return searchString==null ? 
            "%" : '%' + searchString.toLowerCase().replace('*', '%') + '%';
   }
   
   public String getSearchString()
   {
      log.trace("setSearchString");
      return searchString;
   }
   
   public void setSearchString(String searchString)
   {
      log.trace("setSearchString");
      this.searchString = searchString;
   }

   @javax.annotation.PostConstruct
   public void create() {
        log.trace("PostConstruct!");
   }
   
   @Remove
   public void destroy() {}
}
